
/***********************************************************************

  File: simple.c

  Description:
       Interface program of simple advection model

       Usage:
       donor (invoked from AVS read (remote) module, used together
              with interface.c)

  History:

  02/24/93      Gang Cheng      Created 
  04/10/93      Gang Cheng      Modified to seperate simple from
                                interface module

  Copyright NPAC 1993

*************************************************************************/

#include <stdio.h>
#include <math.h>
#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>
#include <sys/types.h> 

#define debug 1
#define sign(x) (((x)>=0.0) ? 1.0:-1.0)
#define max(x,y) (((x)>(y)) ? (x):(y))

extern int simple_f_();
extern int init_f_();

/*MAIN_(argc, argv){ }  for CMfortran Linker */ 

/*main(argc,argv)
int     argc;
char    *argv[];
{
  avs_main(argc,argv);
}
*/

AVSinit_modules()
{
  int simple();
 
  AVSmodule_from_desc(simple);
}

simple()
{
  int simple_compute();     /* declare the compute function (below) */
  int simple_init();        /* declare the initialization function (below) */
  
  int out_port;		/* temporary to hold the port number */
  int parm;		/* temporary to hold the parm number */
  
  /* Set the module name and type */
  AVSset_module_name("Hain(F77)", MODULE_MAPPER);
  
  /* There are no input ports for this module */
  
  /* Create output ports for the result */
  out_port = AVScreate_output_port("Field Output donar", 
				   "field 1D scalar uniform float");
  
  parm = AVSadd_parameter("Grid points","integer", 40, 40, 1000);
  AVSconnect_widget(parm,"idial");
  parm = AVSadd_parameter("Time steps","integer", 100, 50, 1000);
  AVSconnect_widget(parm,"idial");
  parm = AVSadd_parameter("time interval","integer", 0, 0, 1000);
  AVSconnect_widget(parm,"idial");
  
  /* Tell avs what subroutine to call to do the compute */
  AVSset_compute_proc(simple_compute);
  AVSset_init_proc(simple_init);
}

static float fn_simple[1000];

simple_init()
{ 
  register j;
  int pts = 1000;
  
  for( j=0; j<1000; j++) {
    if ((j>=4) && (j<15)) 
	fn_simple[j] =.5 -.4/5.*(float)abs(j+1-10); 
    else
      fn_simple[j] = 0;
  }
  init_f_(&pts);
  fprintf(stderr,"\n init ... \n");
  return;
}

simple_compute(output,points,times,tint)
AVSfield_float **output;
int points,times,tint;
{
  AVSfield_float *temp_field = NULL;

  int i,j;

  float deltt=0.5;
  float buffer; 

  int dims[1];
  static int istep = 0;

  dims[0] = points;

  temp_field = (AVSfield_float *)AVSdata_alloc
    ("field 1D scalar uniform float",dims);
  if (temp_field == NULL) {
    AVSerror("Allocation of output field failed.");
    return(0);
  }
  if (*output) AVSdata_free("field",*output);

  if (tint == 0) {
    for( j=0; j<points; j++) {
      if ((j>=4) && (j<15)) 
	fn_simple[j] = temp_field->data[j] = 
	  .5 -.4/5.*(float)abs(j+1-10); 
      else
	fn_simple[j] = temp_field->data[j] = 0.;
    }

    /* initilize parameter  */
    AVSmodify_parameter("Grid points",AVS_VALUE, 40, 40, 1000);
    AVSmodify_parameter("Time steps",AVS_VALUE, 100, 50, 1000);
    AVSmodify_parameter("time interval",AVS_VALUE, 0, 0, 1000);
    istep = 0;
    *output = temp_field;
    return(1);
  }

  /* fprintf(stderr,"Points = %d times= %d tint = %d\n",points,times,tint); */

  deltt = 50.0/(float)times;

  simple_f_(fn_simple,&points,&deltt,&tint);
  for (i=0; i<points; i++) temp_field->data[i] = fn_simple[i];

  *output = temp_field;
  istep += tint;
  fprintf(stderr,"Simple at time step: %3d\n",istep);

  /* When we're done, we return 1 to indicate success */
  return(1);
} 
